#!/usr/bin/env ruby
#
# Amarok Script for sending amarok music to a freebox
# v 0.3.1
# (c) 2006 Henri Bourcereau <henri.bourcereau@gmail.com>,  Nicolas Morbieu <nicolas.morbieu@gmail.com>
#
# License: GNU General Public License V2

@app_name = "Freebox"

require "net/http"
require "uri"
require "rexml/document"

#Appel dcop  Amarok
def dcop (val)
	str =`dcop amarok player #{val}`
	str.chomp! if str != nil
	return str
end

#criture dans un fichier d'une page xhtml
def write_XHTML(document, file)
	documentStr = String.new
	document.write(documentStr, -1, false, true)
	File.open(file, "w") do |fileWrite|
		fileWrite.write documentStr.gsub('&apos;', '\'')
	end
end

class Freebox
	attr_reader :state
	
	def initialize(scriptDir = File.dirname($0))
		@logfile = 'freebox.log'
		@state = "idle"
		#On copie les fichiers ncessaires au serveur web dans scripts-datas
		`cp #{scriptDir + '/play.html'} #{scriptDir + '/pause.html'} #{scriptDir + '/settings.html'} #{scriptDir + '/Amarok_logo.gif'} .`
		ipFreebox = "212.27.38.253"
		#On lance VLC si ce n'est dj fait
		if `pgrep vlc`==""
			Thread.new {
				system('vlc --extraintf=http --http-host=:8080  --http-charset=ISO-8859-1 --sout="#std" --sout-standard-access=udp --sout-standard-mux=ts --sout-standard-url='+ipFreebox+':1234 --sout-ts-pid-video=68 --sout-ts-pid-audio=69 --sout-ts-pid-spu=70 --sout-ts-pcr=80 --sout-ts-dts-delay=400 --sout-transcode-maxwidth=720 --sout-transcode-maxheight=576  --http-src=.')
			}
			#On attend 3 secondes pour tre sr que VLC a finit de dmarrer
			sleep 3
		end
	end
	
	def quit
		`pkill vlc`
	end
	
	#Joue un morceau de musique
	def play(fichier)
		fichier.strip!
		#~ log "playing "+fichier
		send(fichier, 1)
		@state = "playing"
	end

	def update_informations(informations, coverImage, confPath = File.dirname($0))
		`convert -resize 250 #{coverImage} cover.gif`
		doc = REXML::Document.new File.read(confPath +'/settings-model.xml')
		informations.each_pair do |info_id, info|
			doc.elements["//span[@id='#{info_id}']"].text = info
		end
		write_XHTML(doc, 'settings.html')
	end
	
	def update_lyrics(lyricsXML, confPath = File.dirname($0))
		doc = REXML::Document.new File.read(confPath +'/lyrics-model.xml')
		docLyrics = REXML::Document.new lyricsXML if lyricsXML
		if docLyrics and docLyrics.root and docLyrics.elements
			lyrics = docLyrics.elements['//lyrics'].text if docLyrics.elements['//lyrics']
			artist = docLyrics.root.attributes["artist"]
			title = docLyrics.root.attributes["title"]

			root = REXML::Element.new "div"
			#Remplacement des retours chariots par des br
			if lyrics
				lyrics.each do |line|
					root.add_text line
					root.add_element "br"
				end
			end
			doc.elements["//span[@id='lyrics']"] = root
			doc.elements["//span[@id='artist']"].text = artist if artist
			doc.elements["//span[@id='title']"].text = title if title
		end
		write_XHTML(doc, 'lyrics.html')
	end

	#Affiche une image
	def display(fichier)
		send(fichier, 23)
	end
	
	def pause
		freeboxCall('/pause.html')
		@state = (@state == "playing")?"paused":"playing"
	end
	
	def log(chaine)
		#~ File.open(@logfile, "a") do |fichier|
			#~ fichier << chaine+"\n"
		#~ end
		puts chaine
	end
	
	############# PRIVATE
	private
	
	def send(fichier, type)
		fichier.gsub!(/\+/, '%2B')
		fichier.gsub!(/&/, '%26')
		fichier = URI.escape(fichier).gsub(/'/, '%5C%27')
		puts fichier
		freeboxCall('/play.html?type='+type.to_s+'&mrl='+fichier)
	end
	
	def freeboxCall(url)
		#~ log "Calling URL "+url
		Net::HTTP.start( '127.0.0.1', 8080 ) do |http|
			http.get( url)
		end
	end
end

##################################################################
# MAIN
##################################################################

freebox = Freebox.new(File.dirname(File.expand_path( __FILE__) ) )

trap( "SIGTERM" ) {
	freebox.close
}

loop do
    message = gets().chomp()
    command = /[A-Za-z]*/.match( message ).to_s()

	case command
		when "trackChange"
			fileToPlay = dcop('path')
			coverImage = dcop('coverImage')
			lyrics = dcop('lyrics')

			info_keys = ['title', 'artist', 'album']
			informations = Hash.new
			info_keys.each do |info_key|
				informations[info_key] = dcop(info_key)
			end
			freebox.update_informations(informations, coverImage)
			freebox.update_lyrics(lyrics)
			freebox.play(fileToPlay)
			
		when "engineStateChange"
			#freebox.log message
			case message
				when "engineStateChange: paused"
					freebox.pause if freebox.state == "playing"
				when "engineStateChange: playing"
					freebox.pause if freebox.state == "paused"
			end
    end
end

